// Copyright (c) 2016 Citrix Systems, Inc. All Rights Reserved.
// You may only reproduce, distribute, perform, display, or prepare derivative works of this file pursuant to a valid license from Citrix. 
// THIS SAMPLE CODE IS PROVIDED BY CITRIX "AS IS" AND ANY EXPRESS OR IMPLIED 
// WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 

#pragma once

#ifdef __cplusplus
namespace CitrixAuthManagerCustomProtocolSDK
{
// Ensure C function name usage for C++ compilers.
extern "C" {
#endif

// This is the native character type for your platform, using the native unicode encoding.
typedef wchar_t AMChar;

// Client process' window information.
typedef struct AMWindowInfo_tag
{
    int bottom;             // The bottom coordinate of the hosting window.
    int left;               // The left coordinate of the hosting window.
    int width;              // The width of the hosting window.
    int height;             // The height of the hosting window.
    AMChar* windowName;		// The name of the hosting window.
    AMChar* windowID;		// There's no code that ever sets this.
    AMChar* amWindowName;	// Name to be used for the AuthManager UI window.
    void* advancedInfo;		// Pointer to more info used as of V3; there is no code that ever sets this.
} AMWindowInfo;

// Return codes.
typedef enum eAuthResult_tag {
    Success = 0,
    AccessDenied  = 1,
    CancelledByUser = 2,
    SystemError = 3,
    NetworkError = 4,
    InteractionNotAllowed = 5,
    DisallowedBySecurityPolicy = 6,
    AuthenticationServerError = 7,
    UnsuitableProtocol = 8,
    ProtocolError = 9
} eAuthResult;

// Authentication behaviour flags.
typedef enum eAuthFlags_tag {
    None = 0,
    Interactive = 1,
} eAuthFlags;

// Note that the strings passed into this interface are all UTF8 byte format strings
// rather than single byte ASCII strings 
typedef char* UTF8ByteArray;

// This function is called on your DLL by Auth Manager to perform authentication with a paired StoreFront authentication extension.
// This plugin interface cannot be used in conjunction with gateways, only direct access to StoreFront.

// The return code is an eAuthResult to inform Auth Manager of the authentication success or failure reason.See AMLogonResult.h
eAuthResult CitrixAuthenticationManager_Authenticate(UTF8ByteArray startURL,               // The UTF8 URL to contact on the StoreFront server. This is null terminated.
                                                     UTF8ByteArray tokenRequest,           // The UTF8 token request that the authentication URL will need to be passed. This is null terminated.
                                                     eAuthFlags flags,                     // An eAuthFlags enum. If the authentication process is allowed to show UI this is Interactive otherwise None.
                                                     const AMWindowInfo& clientWindowInfo, // This contains information about the client process' window.
                                                     UTF8ByteArray* outAuthenticationData, // The UTF8 token response data from StoreFront to pass back to Auth Manager.
                                                     size_t* outDataSize);                 // The size of the token response data in bytes.

// Call this to free the data returned by Authenticate.
void CitrixAuthenticationManager_Free(void* authenticationData);

// End of C function name usage scope.
#ifdef __cplusplus
} // End extern C
} // End of namespace.
#endif
